﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Reflection;
using System.CodeDom.Compiler;
using Microsoft.JScript;
using System.Collections;
using System.Threading;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Evaluates expressions at runtime.
	/// </summary>
	public class Evaluator
	{

		/// <summary>
		/// Represents standard handler that is called to evaluate expression.
		/// </summary>
		/// <returns>Expression evaluation result.</returns>
		public delegate object EvalDelegate();
		
		/// <summary>
		/// Represents custom eval handler that is called when expression is evaluated.
		/// </summary>
		/// <param name="evalDelegate">Standard handler that evaluates an expression.</param>
		/// <returns>Expression evaluation result.</returns>
		public delegate object CustomEvalHandlerDelegate(EvalDelegate evalDelegate);

		private static Dictionary<int, CustomEvalHandlerDelegate> customEvalHandlers = new Dictionary<int, CustomEvalHandlerDelegate>();
		private static object innerEvaluator = null;
		private static MethodInfo evalMethod = null;

		private static string EvaluatorSource = @"
		import System;
		
		package Mesware.MesControl.Model
		{
			class InnerEvaluator
			{
				var utils: Object;

				public function Eval(expr: String, params: Object[]): Object
				{
					return eval(expr, ""unsafe"");
				}
				
				function InnerEvaluator(utils: Object)
				{
					this.utils = utils;
				}
			}
		}
		";

		internal static void Initialize()
		{
			CompilerParameters options = new CompilerParameters();
			options.GenerateInMemory = true;

			Assembly[] assemblies = AppDomain.CurrentDomain.GetAssemblies();
			foreach(Assembly assembly in assemblies) {
				options.ReferencedAssemblies.Add(assembly.Location);
			}

			JScriptCodeProvider codeProvider = new JScriptCodeProvider();
			CompilerResults results = codeProvider.CompileAssemblyFromSource(options, EvaluatorSource);

			if (results.Errors.HasErrors) {
				StringBuilder sb = new StringBuilder();
				sb.Append("EvaluatorCompilationFailed");
				foreach (CompilerError error in results.Errors)
					sb.AppendFormat(" {0}: {1}.", error.ErrorNumber, error.ErrorText);
				throw new Exception(sb.ToString());
			}

			System.Type evaluatorType = results.CompiledAssembly.GetType("Mesware.MesControl.Model.InnerEvaluator");

			innerEvaluator = Activator.CreateInstance(evaluatorType, new EvaluatorUtils());

			evalMethod = evaluatorType.GetMethod("Eval");
		}
		
		/// <summary>
		/// Evaluates an expression without parameters.
		/// </summary>
		/// <param name="expression">Expression.</param>
		/// <returns>Expression evaluation result.</returns>
		public static object Eval(string expression)
		{
			return Eval(expression, null);
		}
		
		/// <summary>
		/// Evaluates an expression using specified named parameters.
		/// </summary>
		/// <param name="expression">Expression.</param>
		/// <param name="parameters">Named parameters.</param>
		/// <returns>Expression evaluation result.</returns>
		public static object Eval(string expression, Hashtable parameters)
		{
			if (expression==null)
				throw new ArgumentNullException("expression");
				
			object[] evalParams = null;
			
			if (parameters!=null) {
				ArrayList evalParamsList = new ArrayList();
				foreach (DictionaryEntry entry in parameters) {
					string parameterName = "{" + (string)entry.Key + "}";
					string parameterAlias = "params[" + evalParamsList.Count + "]";
					expression = expression.Replace(parameterName, parameterAlias);
					evalParamsList.Add(entry.Value);
				}
				evalParams = evalParamsList.ToArray();
			}
			
			CustomEvalHandlerDelegate customEvalHandler = null;

			lock (customEvalHandlers) {
				int threadId = Thread.CurrentThread.ManagedThreadId;
				if (customEvalHandlers.ContainsKey(threadId))
					customEvalHandler = customEvalHandlers[threadId];
			}

			if (customEvalHandler!=null) {
				EvalDelegate evalDelegate = delegate {
					return evalMethod.Invoke(innerEvaluator, new object[] { expression, evalParams });
				};
				return customEvalHandler(evalDelegate);
			}

			return evalMethod.Invoke(innerEvaluator, new object[] {expression, evalParams});
		}
		
		/// <summary>
		/// Gets or sets custom eval handler associated with the current thread.
		/// </summary>
		public static CustomEvalHandlerDelegate CustomEvalHandler {
			get {
				lock(customEvalHandlers) {
					int threadId = Thread.CurrentThread.ManagedThreadId;
					if (customEvalHandlers.ContainsKey(threadId))
						return customEvalHandlers[threadId];
					return null;
				}
			}
			set {
				lock (customEvalHandlers) {
					int threadId = Thread.CurrentThread.ManagedThreadId;
					if (customEvalHandlers.ContainsKey(threadId) && value==null)
						customEvalHandlers.Remove(threadId);
					else
						customEvalHandlers[threadId] = value;
				}
			}
		}
	}
}
