using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Common;
using System.Data;
using System.Collections;
using System.Text.RegularExpressions;
using System.Reflection;
using Type = System.Type;
using System.Collections.Specialized;
using Mesware.MesControl.DataProvider.Properties; 

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents an OQL statement to execute against a MEScontrol.
	/// </summary>
	public class MEScontrolCommand : DbCommand
	{
		private string commandText;
		private MEScontrolConnection connection;
		private bool designTimeVisible;
		private UpdateRowSource updateRowSource;
		private MEScontrolParameterCollection parameters = new MEScontrolParameterCollection();
		private List<string> expressions = new List<string>();
		private Dictionary<string, Type> typeByExpression = new Dictionary<string, Type>();
		private object targetObject;

		public object TargetObject
		{
			get { return targetObject; }
			set { targetObject = value; }
		}

		/// <summary>
		/// Gets or sets the text command to run.
		/// </summary>
		public override string CommandText {
			get {	return commandText; }
			set { commandText = value; }
		}

		/// <summary>
		/// Gets or sets the wait time before terminating the attempt to execute a command and generating an error.
		/// Only value 0 is supported.
		/// </summary>
		public override int CommandTimeout {
			get { return 0; }
			set { if (value!=0) throw new NotSupportedException(TranslationResources.OnlyCommandTimeout0ValueIsSupported); }
		}

		/// <summary>
		/// Indicates or specifies how the <see cref="CommandText"/> property is interpreted.
		/// Only CommandType.Text is supported.
		/// </summary>
		public override CommandType CommandType {
			get { return CommandType.Text; }
			set { if (value!=CommandType.Text) throw new NotSupportedException(TranslationResources.OnlyCommandTypeTextIsSupported); }
		}

		/// <summary>
		/// Gets or sets the <see cref="DbConnection"/> used by this <see cref="MEScontrolCommand"/>. 
		/// </summary>
		protected override DbConnection DbConnection {
			get { return connection; }
			set	{
				if (!(value is MEScontrolConnection))
					throw new NotSupportedException(TranslationResources.OnlyMEScontrolConnectionIsSupported);
				if (connection!=value)
					Transaction = null;
				connection = (MEScontrolConnection)value;
			}
		}

		/// <summary>
		/// Gets or sets the <see cref="DbTransaction"/> within which this 
		/// <see cref="MEScontrolCommand"/> object executes. 
		/// </summary>
		protected override DbTransaction DbTransaction
		{
			get { return null; }
			set {  }
		}

		/// <summary>
		/// Hidden property used by the designers.
		/// </summary>
		public override bool DesignTimeVisible {
			get { return designTimeVisible; }
			set { designTimeVisible = value; }
		}

		/// <summary>
		/// Gets or sets how command results are applied to the <see cref="DataRow"/>
		/// when used by the Update method of a <see cref="DbDataAdapter"/>.
		/// </summary>
		public override UpdateRowSource UpdatedRowSource {
			get { return updateRowSource; }
			set { updateRowSource = value; }
		}

		/// <summary>
		/// Gets the collection of <see cref="DbParameter"/> objects.
		/// </summary>
		protected override DbParameterCollection DbParameterCollection {
			get { return parameters; }
		}

		/// <summary>
		/// Gets the collection of <see cref="MEScontrolParameter"/> objects.
		/// </summary>
		public new MEScontrolParameterCollection Parameters {
			get { return parameters; }
		}

		/// <summary>
		/// Attempts to cancels the execution of a <see cref="MEScontrolCommand"/>.
		/// </summary>
		public override void Cancel()
		{			
		}

		/// <summary>
		/// Creates a prepared (or compiled) version of the command on the data source.
		/// </summary>
		public override void Prepare()
		{
		}

		/// <summary>
		/// Adds new field to <see cref="MEScontrolDataReader"/>.
		/// </summary>
		/// <param name="name">New field name.</param>
		public void AddField(string name, Type type)
		{
			expressions.Add(name);
			typeByExpression.Add(name, type); 
		}

		/// <summary>
		/// Executes a OQL statement against a connection object.
		/// </summary>
		/// <returns>The number of rows returned.</returns>
		public override int ExecuteNonQuery()
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// Executes the query and returns the first column of the first
		/// row in the result set returned by the query. All other columns and rows are ignored.
		/// </summary>
		/// <returns>The first column of the first row in the result set.</returns>
		public override object ExecuteScalar()
		{
			throw new NotImplementedException();
		}

		/// <summary>
		/// Creates a new instance of a <see cref="DbParameter"/> object. 
		/// </summary>
		/// <returns>A <see cref="DbParameter"/> object.</returns>
		protected override DbParameter CreateDbParameter()
		{
			return CreateParameter();
		}

		/// <summary>
		/// Creates a new instance of a <see cref="MEScontrolParameter"/> object. 
		/// </summary>
		/// <returns>A <see cref="MEScontrolParameter"/> object.</returns>
		public new MEScontrolParameter CreateParameter()
		{
			return new MEScontrolParameter();
		}

		/// <summary>
		/// Executes the command text against the connection.
		/// </summary>
		/// <param name="behavior">An instance of <see cref="CommandBehavior"/>.</param>
		/// <returns>A <see cref="DbDataReader"/>.</returns>
		protected override DbDataReader ExecuteDbDataReader(CommandBehavior behavior)
		{
			return ExecuteReader(behavior);
		}

		/// <summary>
		/// Executes the <see cref="CommandText"/> against the <see cref="DbConnection"/>,
		/// and returns an <see cref="MEScontrolDataReader"/>.
		/// </summary>
		/// <returns>A <see cref="MEScontrolDataReader"/>.</returns>
		public new MEScontrolDataReader ExecuteReader()
		{
			return ExecuteReader(CommandBehavior.Default);  
		}

		/// <summary>
		/// Executes the <see cref="CommandText"/> against the <see cref="DbConnection"/>,
		/// and returns an <see cref="MEScontrolDataReader"/> using one of the <see cref="CommandBehavior"/> values.
		/// </summary>
		/// <param name="behavior">An instance of <see cref="CommandBehavior"/>.</param>
		/// <returns>A <see cref="MEScontrolDataReader"/>.</returns>
		public new MEScontrolDataReader ExecuteReader(CommandBehavior behavior)
		{			
			if (connection==null || connection.State!=ConnectionState.Open)
				throw new InvalidOperationException(TranslationResources.ConnectionMustValidAndOpen);
			if (commandText==null)
				throw new InvalidOperationException(TranslationResources.CommandTextMustBeNotNull);

			// TODO: move check logic to IDataReaderSource
			IDataReaderSource source = null;
			string trimmedText = commandText.Trim();
			int endOfFirstToken = trimmedText.IndexOf(' ');
			if (endOfFirstToken==-1)
				endOfFirstToken = trimmedText.Length; 
			string firstToken = trimmedText.Substring(0, endOfFirstToken);
			
			string partBeforeBracket = trimmedText.Split('(')[0];
			if (partBeforeBracket.Contains(".") || targetObject==null)
				source = new ServiceDataReaderSource(commandText);
			else
				source = new SpecifiedObjectDataReaderSource(commandText, targetObject);
			

			source.SetParameters(Parameters);
			MEScontrolDataReader reader = new MEScontrolDataReader(source, behavior, typeByExpression);
			return reader;

		}



		private ArrayList ParseParameters(string query, char parameterPrefix)
		{
			ArrayList list = new ArrayList();
			if (query.Length!=0) {
				string text = @"\p{Ll}\p{Lu}\p{Lt}\p{Lm}\p{Lo}\p{Nl}\p{Nd}\p{Mn}\p{Mc}\p{Pc}\p{Cf}";
				string text2 = string.Format("([^{0}?\"'[/-]|(?<=[{1}{0}])[{0}?]|[{0}](?![{1}])|/(?!\\*)|-(?!-))", parameterPrefix, text);
				string text3 = "\"([^\"]|\"\")*\"";
				string text4 = "'([^']|'')*'";
				string text5 = string.Format("({0}|{1})", text3, text4);
				string text6 = @"\[[^\[\]]*\]";
				string text7 = @"/\*(.|\n)*\*/";
				string text8 = "--.*$";
				string text9 = string.Format("({0}|{1})", text7, text8);
				string text10 = string.Format("({0}|{1}|{2})", text5, text6, text9);
				string text11 = string.Format(@"(?<=\G({1}*{2})*({1}*[^{0}{3}]|))", new object[] { text, text2, text10, parameterPrefix });
				MatchCollection matchs = 
					new Regex(string.Format(@"{2}([{0}][{1}]+|\?)", parameterPrefix, text, text11), RegexOptions.Multiline).Matches(query);
				for (int i=0;i<matchs.Count;i++) {
					string item = matchs[i].Value;
					if (!list.Contains(item))					
						list.Add(item);					
				}
			}
			return list;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="MEScontrolCommand"/> class with the text of the query.
		/// </summary>
		/// <param name="cmdText">The text of the query.</param>
		public MEScontrolCommand(string cmdText)
			: this(cmdText, null, null)
		{
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="MEScontrolCommand"/> class
		/// with the text of the query and a <see cref="MEScontrolConnection"/>.
		/// </summary>
		/// <param name="cmdText">The text of the query.</param>
		/// <param name="connection">A <see cref="MEScontrolConnection"/> that represents
		/// the connection to a MEScontrol.</param>
		public MEScontrolCommand(string cmdText, MEScontrolConnection connection)
			: this(cmdText, connection, null)
		{			
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="MEScontrolCommand"/> class
		/// with the text of the query and a <see cref="MEScontrolConnection"/>.
		/// </summary>
		/// <param name="cmdText">The text of the query.</param>
		/// <param name="connection">A <see cref="MEScontrolConnection"/> that represents
		/// the connection to a MEScontrol.</param>
		public MEScontrolCommand(string cmdText, MEScontrolConnection connection, object targetObject)
		{
			CommandText = cmdText;
			Connection = connection;
			this.targetObject = targetObject;
		}	
	}
}
