using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Common;
using System.Data;
using System.Runtime.Remoting;
using System.Reflection;
using System.IO;
using System.Security;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents a connection to a MEScontrol.
	/// </summary>
	public class MEScontrolConnection : DbConnection
	{
		private ConnectionState state = ConnectionState.Closed;
		private string userName;
		private string password;
		private string connectionString;

		/// <summary>
		/// Gets or sets the string used to open the connection.
		/// </summary>
		public override string ConnectionString {
			get { return connectionString; }
			set {
				connectionString = value;
				Dictionary<string, string> vars = GetConnectionStringVars(connectionString);
				if (vars.ContainsKey("UserName"))
					userName = vars["UserName"];
				if (vars.ContainsKey("Password"))
					password = vars["Password"];
			}
		}

		/// <summary>
		/// Gets the name of the database server to which to connect. Is not used.
		/// </summary>
		public override string DataSource {
			get { return ""; }
		}

		/// <summary>
		/// Gets the name of the current database after a connection is opened,
		/// or the database name specified in the connection string before the 
		/// connection is opened. Is not used.
		/// </summary>
		public override string Database {
			get { return ""; }
		}

		/// <summary>
		/// Gets a string that represents the version of the server to which the object
		/// is connected. Is not used.
		/// </summary>
		public override string ServerVersion {
			get { return ""; }
		}

		/// <summary>
		/// Gets the current state of the connection.
		/// </summary>
		public override ConnectionState State	{
			get { return state; }
		}

		private static Dictionary<string, string> GetConnectionStringVars(string connectionString)
		{
			Dictionary<string, string> vars = new Dictionary<string, string>(StringComparer.InvariantCultureIgnoreCase);   
			string[] definitions = connectionString.Split(";".ToCharArray());
			foreach (string definition in definitions) {
				string[] namevalue = definition.Split("=".ToCharArray());
				if (namevalue.Length > 1)
					vars.Add(namevalue[0].Trim(), namevalue[1].Trim());
			}
			return vars;
		}
		
		/// <summary>
		/// Opens a database connection with the settings specified by the
		/// <see cref="ConnectionString"/>.
		/// </summary>
		public override void Open()
		{
			if (state==ConnectionState.Open)
				throw new InvalidOperationException(TranslationResources.TheConnectionIsAlreadyOpen);

			state = ConnectionState.Open; 
		}

		/// <summary>
		/// Closes the connection to the database.
		/// This is the preferred method of closing any open connection. 
		/// </summary>
		public override void Close()
		{
			if (state==ConnectionState.Closed)
				throw new InvalidOperationException(TranslationResources.TheConnectionIsAlreadyClosed);
			state = ConnectionState.Closed;
		}


		/// <summary>
		/// Creates and returns a <see cref="DbCommand"/> object associated with the current connection.
		/// </summary>
		/// <returns>A <see cref="DbCommand"/> object.</returns>
		protected override DbCommand CreateDbCommand()
		{
			return new MEScontrolCommand(null, this);
		}

		/// <summary>
		/// Creates and returns a <see cref="MEScontrolCommand"/> object associated with the current connection.
		/// </summary>
		/// <returns>A <see cref="MEScontrolCommand"/> object.</returns>
		public new MEScontrolCommand CreateCommand()
		{
			return new MEScontrolCommand(null, this);
		}

		/// <summary>
		/// Starts a database transaction.
		/// </summary>
		/// <param name="isolationLevel">Specifies the isolation level for the transaction.</param>
		/// <returns>An object representing the new transaction.</returns>
		protected override DbTransaction BeginDbTransaction(IsolationLevel isolationLevel)
		{
			return null;
		}

		/// <summary>
		/// Changes the current database for an open connection.
		/// Not supported.
		/// </summary>
		/// <param name="databaseName">Specifies the name of the database for the connection to use.</param>
		public override void ChangeDatabase(string databaseName)
		{
			throw new NotSupportedException(TranslationResources.ThisMethodIsNotSupported);
		}


		/// <summary>
		/// Initializes a new instance of the <see cref="MEScontrolConnection"/> class. 
		/// </summary>
		/// <param name="connectionString">The connection used to open connection.</param>
		public MEScontrolConnection(string connectionString)
			: this()
		{
			ConnectionString = connectionString;
		}

		/// <summary>
		/// Initializes a new instance of the <see cref="MEScontrolConnection"/> class. 
		/// </summary>
		public MEScontrolConnection()
			: base()
		{			
		}

	}
}
