using System;
using System.Collections;
using System.Collections.Generic;
using System.Data;
using System.Data.Common;
using System.Reflection;
using System.Text;
using Type = System.Type;
using System.Globalization;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Reads a forward-only stream of rows from a data source.
	/// </summary>
	public class MEScontrolDataReader : DbDataReader
	{
		private IEnumerable data;
		private IEnumerator enumerator;
		private IDataReaderSource source;
		private bool opened = true;
		private List<Type> forbiddenTypes = new List<Type>(2);
		private List<string> expressions = new List<string>();
		private Dictionary<string, Type> typeByField = new Dictionary<string, Type>(); 
		private Type targetType;
		private CommandBehavior behavior;

		/// <summary>
		/// Gets a value indicating the depth of nesting for the current row.
		/// Always 0 for <see cref="MEScontrolDataReader"/>.
		/// </summary>
		public override int Depth {
			get { return 0; }
		}

		/// <summary>
		/// Gets the number of columns in the current row.
		/// </summary>
		public override int FieldCount {
			// TODO: ???
			get {
				if (expressions.Count==0 && behavior!=CommandBehavior.SchemaOnly)  
					return 1000;
				return expressions.Count;
			}
		}

		/// <summary>
		/// Gets a value that indicates whether this <see cref="MEScontrolDataReader"/> contains one or more rows.
		/// </summary>
		public override bool HasRows {
			get {
				bool hasRows = false;
				foreach (object obj in data) {
					hasRows = true;
					break;
				}
				return hasRows;
			}
		}

		/// <summary>
		/// Gets a value indicating whether the <see cref="MEScontrolDataReader"/> is closed.
		/// </summary>
		public override bool IsClosed {
			get { return !opened; }
		}

		/// <summary>
		/// Gets a value indicating whether the <see cref="MEScontrolDataReader"/> contains one or more rows. 
		/// </summary>
		public override int RecordsAffected {
			get { return -1; }
		}

		/// <summary>
		/// Gets the value of the specified column as an instance of <see cref="Object"/>.
		/// </summary>
		/// <param name="name">The name of the column.</param>
		/// <returns>The value of the specified column.</returns>
		public override object this[string name] {
			get {
				return GetValue(GetOrdinal(name));
			}
		}

		/// <summary>
		/// Gets the value of the specified column as an instance of <see cref="Object"/>.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override object this[int ordinal] {
			get { return GetValue(ordinal); }
		}

		private static int CompareMemberInfo(MemberInfo x, MemberInfo y)
		{
			return string.Compare(x.Name, y.Name);  
		}

		private MemberInfo[] PopulateMembers(System.Type type)
		{
			List<MemberInfo> members = new List<MemberInfo>();
			MemberInfo[] allMembers = type.GetMembers(BindingFlags.Instance | BindingFlags.Public);
			 
			foreach (MemberInfo member in allMembers) {
				PropertyInfo prop = member as PropertyInfo;
				FieldInfo field = member as FieldInfo;

				if (prop==null && field==null)
					continue; 
				
				if (prop!=null && !prop.CanRead)
					continue;


				if (prop!=null && prop.GetIndexParameters().Length>0)
					continue;

				bool forbidden = false;
				Type returnType = prop!=null ? prop.PropertyType : field.FieldType;
				foreach(Type forbiddenType in forbiddenTypes)
					if (returnType==forbiddenType || returnType.IsSubclassOf(forbiddenType)) {
						forbidden = true;
						break; 
					}
				if (forbidden)
					continue;

				members.Add(member);				
				
			}
			members.Sort(CompareMemberInfo); 
			return members.ToArray(); 
		}

		/// <summary>
		/// Closes the <see cref="MEScontrolDataReader"/> object.
		/// </summary>
		public override void Close()
		{
			opened = false;
		}

		/// <summary>
		/// Gets the value of the specified column as a Boolean. 
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override bool GetBoolean(int ordinal)
		{
			return (bool)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a Boolean. 
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override byte GetByte(int ordinal)
		{
			return (byte)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a single character. 
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override char GetChar(int ordinal)
		{
			return (char)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a <see cref="DateTime"/> object.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override DateTime GetDateTime(int ordinal)
		{
			return (DateTime)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a <see cref="Decimal"/> object.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override decimal GetDecimal(int ordinal)
		{
			return (decimal)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a double-precision floating point number.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override double GetDouble(int ordinal)
		{
			return (double)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a single-precision floating point number.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override float GetFloat(int ordinal)
		{
			return (float)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a globally-unique identifier (GUID).
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override Guid GetGuid(int ordinal)
		{
			return (Guid)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a 16-bit signed integer.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override short GetInt16(int ordinal)
		{
			return (short)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a 32-bit signed integer.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override int GetInt32(int ordinal)
		{
			return (int)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as a 64-bit signed integer.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override long GetInt64(int ordinal)
		{
			return (long)GetValue(ordinal);
		}

		/// <summary>
		/// Gets the value of the specified column as an instance of <see cref="String"/>.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override string GetString(int ordinal)
		{
			return (string)GetValue(ordinal);
		}

		/// <summary>
		/// Reads a stream of bytes from the specified column,
		/// starting at location indicated by dataIndex, into the buffer, starting at the location indicated by bufferIndex.
		/// Not supported by <see cref="MEScontrolDataReader"/>.
		/// </summary>		
		public override long GetBytes(int ordinal, long dataOffset, byte[] buffer, int bufferOffset, int length)
		{
			throw new NotSupportedException(TranslationResources.TheMethodIsNotSupported);
		}		

		/// <summary>
		/// Reads a stream of characters from the specified column, starting at location indicated by dataIndex,
		/// into the buffer, starting at the location indicated by bufferIndex.
		/// Not supported by <see cref="MEScontrolDataReader"/>.
		/// </summary>
		public override long GetChars(int ordinal, long dataOffset, char[] buffer, int bufferOffset, int length)
		{
			throw new NotSupportedException(TranslationResources.TheMethodIsNotSupported);
		}

		/// <summary>
		/// Gets name of the data type of the specified column. 
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>A string representing the name of the data type.</returns>
		public override string GetDataTypeName(int ordinal)
		{
			return GetFieldType(ordinal).Name;  
		}

		/// <summary>
		/// Returns an <see cref="IEnumerator"/> that can be used to iterate through
		/// the rows in the data reader.
		/// </summary>
		/// <returns>An <see cref="IEnumerator"/> that can be used to iterate through
		/// the rows in the data reader.</returns>
		public override IEnumerator GetEnumerator()
		{
			return new DbEnumerator(this, true);
		}

		/// <summary>
		/// Gets the data type of the specified column.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The data type of the specified column.</returns>
		public override System.Type GetFieldType(int ordinal)
		{
			if (!opened)
				throw new InvalidOperationException(TranslationResources.MEScontrolDataReaderIsClosed);
			
			string expression = GetName(ordinal);
			if (typeByField!=null && typeByField.ContainsKey(expression))
				return typeByField[expression];

			string[] parts = expression.Split('.');
			Type type = targetType;

			foreach (string propertyName in parts) {
				if (propertyName.ToLowerInvariant()=="this")
					break;
				MemberInfo[] members = 
						GetMember(type, propertyName, MemberTypes.Property | MemberTypes.Field, BindingFlags.Instance | BindingFlags.Public);
				if (members.Length==0)
					return typeof(object);
				PropertyInfo prop = members[0] as PropertyInfo;
				if (prop!=null)
					type = prop.PropertyType;
				else {
					FieldInfo field = members[0] as FieldInfo;
					if (field!=null)
						type = field.FieldType;
				}
			}

			return type;
		}

		private MemberInfo[] GetMember(Type type, string propertyName, MemberTypes memberType, BindingFlags bindingFlags)
		{
			MemberInfo[] members = type.GetMember(propertyName, memberType, bindingFlags);
			List<MemberInfo> result = new List<MemberInfo>(members);
			if (type.IsInterface)
				foreach (Type parentInterface in type.GetInterfaces())
					result.AddRange(GetMember(parentInterface, propertyName, memberType, bindingFlags));
			return result.ToArray();
		}

		/// <summary>
		/// Gets the name of the column, given the zero-based column ordinal.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The name of the specified column.</returns>
		public override string GetName(int ordinal)
		{
			if (!opened)
				throw new InvalidOperationException(TranslationResources.MEScontrolDataReaderIsClosed);
			return expressions[ordinal];
		}

		/// <summary>
		/// Gets the column ordinal given the name of the column.
		/// </summary>
		/// <param name="name">The name of the column.</param>
		/// <returns>The zero-based column ordinal.</returns>
		public override int GetOrdinal(string name)
		{
			if (!opened)
				throw new InvalidOperationException(TranslationResources.MEScontrolDataReaderIsClosed);
			for (int i=0; i<expressions.Count; i++) {
				if (expressions[i]==name)
					return i;
			}
			expressions.Add(name);
			return expressions.Count-1;
		}

		private DataTable NewSchemaTable()
		{
			DataTable table = new DataTable("SchemaTable");
			table.Locale = CultureInfo.InvariantCulture;
			table.MinimumCapacity = this.FieldCount;
			DataColumnCollection columns = table.Columns;
			columns.Add(new DataColumn("ColumnName", typeof(string)));
			columns.Add(new DataColumn("ColumnOrdinal", typeof(int)));
			columns.Add(new DataColumn("ColumnSize", typeof(int)));
			columns.Add(new DataColumn("NumericPrecision", typeof(short)));
			columns.Add(new DataColumn("NumericScale", typeof(short)));
			columns.Add(new DataColumn("DataType", typeof(object)));
			columns.Add(new DataColumn("ProviderType", typeof(int)));
			columns.Add(new DataColumn("IsLong", typeof(bool)));
			columns.Add(new DataColumn("AllowDBNull", typeof(bool)));
			columns.Add(new DataColumn("IsReadOnly", typeof(bool)));
			columns.Add(new DataColumn("IsRowVersion", typeof(bool)));
			columns.Add(new DataColumn("IsUnique", typeof(bool)));
			columns.Add(new DataColumn("IsKey", typeof(bool)));
			columns.Add(new DataColumn("IsAutoIncrement", typeof(bool)));
			columns.Add(new DataColumn("BaseSchemaName", typeof(string)));
			columns.Add(new DataColumn("BaseCatalogName", typeof(string)));
			columns.Add(new DataColumn("BaseTableName", typeof(string)));
			columns.Add(new DataColumn("BaseColumnName", typeof(string)));
			foreach (DataColumn column in columns)
					column.ReadOnly = true;
			return table;
		} 


		/// <summary>
		/// Returns a <see cref="DataTable"/> that describes the column metadata
		/// of the <see cref="MEScontrolDataReader"/>.
		/// Not supported by <see cref="MEScontrolDataReader"/>.
		/// </summary>
		/// <returns>A <see cref="DataTable"/> that describes the column metadata.</returns>
		public override DataTable GetSchemaTable()
		{
			if (!opened)
				throw new InvalidOperationException(TranslationResources.MEScontrolDataReaderIsClosed);

			DataTable table = this.NewSchemaTable();
			if (this.FieldCount!=0) {
				DataColumn column18 = table.Columns["ColumnName"];
				DataColumn column17 = table.Columns["ColumnOrdinal"];
				DataColumn column16 = table.Columns["ColumnSize"];
				DataColumn column15 = table.Columns["NumericPrecision"];
				DataColumn column14 = table.Columns["NumericScale"];
				DataColumn column13 = table.Columns["DataType"];
				DataColumn column12 = table.Columns["ProviderType"];
				DataColumn column11 = table.Columns["IsLong"];
				DataColumn column10 = table.Columns["AllowDBNull"];
				DataColumn column9 = table.Columns["IsReadOnly"];
				DataColumn column8 = table.Columns["IsRowVersion"];
				DataColumn column7 = table.Columns["IsUnique"];
				DataColumn column6 = table.Columns["IsKey"];
				DataColumn column5 = table.Columns["IsAutoIncrement"];
				DataColumn column4 = table.Columns["BaseSchemaName"];
				DataColumn column3 = table.Columns["BaseCatalogName"];
				DataColumn column2 = table.Columns["BaseTableName"];
				DataColumn column = table.Columns["BaseColumnName"];
				int fieldCount = this.FieldCount;
				for (int i=0; i<fieldCount; i++) {
					DataRow row = table.NewRow();
					row[column18] = this.GetName(i);
					row[column17] = i;
					row[column16] = 0;
					row[column15] = 0;
					row[column14] = 0;
					row[column13] = this.GetFieldType(i);
					row[column12] = 0;
					row[column11] = false;
					row[column10] = true;
					row[column9] = true;
					row[column8] = false;
					row[column7] = false;
					row[column6] = false;
					row[column5] = false;
					row[column4] = "";
					row[column3] = "";
					row[column2] = "";
					row[column] = "";
					table.Rows.Add(row);
					row.AcceptChanges();
				}
			}
			return table;

		}

		/// <summary>
		/// Gets the value of the specified column as an instance of <see cref="Object"/>.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns>The value of the specified column.</returns>
		public override object GetValue(int ordinal)
		{
			if (!opened)
				throw new InvalidOperationException(TranslationResources.MEScontrolDataReaderIsClosed);			

			object obj = enumerator.Current;
			if (obj==null)
				return null;

			string[] parts = expressions[ordinal].Split('.'); 			
						
			object target = obj;		
			foreach (string propertyName in parts) {
				if (propertyName.ToLowerInvariant()=="this")				
					break;
				List<MemberInfo> members = new List<MemberInfo>(  
					target.GetType().GetMember(propertyName, MemberTypes.Property, BindingFlags.Instance | BindingFlags.Public));
				members.AddRange(
					target.GetType().GetMember(propertyName, MemberTypes.Field, BindingFlags.Instance | BindingFlags.Public)); 
				if (members.Count==0)
					throw new InvalidOperationException(string.Format(
						TranslationResources.TypeHaveNoPropertyWithName,
						target.GetType().Namespace+"."+target.GetType().Name,
						propertyName));
				PropertyInfo prop = members[0] as PropertyInfo;
				FieldInfo field = members[0] as FieldInfo;
				if (prop!=null)
					target = prop.GetValue(target, null);
				else
					target = field.GetValue(target);
			}

			return target;
		}

		/// <summary>
		/// Gets all attribute columns in the collection for the current row. 
		/// </summary>
		/// <param name="values">An array of <see cref="Object"/> into which to
		/// copy the attribute columns.</param>
		/// <returns>The number of instances of <see cref="Object"/> in the array.</returns>
		public override int GetValues(object[] values)
		{
			int i=0;
			for (;i<values.Length && i<expressions.Count;i++)
				values[i] = GetValue(i);			
			return i;
		}		

		/// <summary>
		/// Gets a value that indicates whether the column contains nonexistent or missing values.
		/// </summary>
		/// <param name="ordinal">The zero-based column ordinal.</param>
		/// <returns><see langword="true"/> if the specified column is equivalent to DBNull; 
		/// otherwise <see langword="false"/>.</returns>
		public override bool IsDBNull(int ordinal)
		{
			return GetValue(ordinal)==null;
		}

		/// <summary>
		/// Advances the reader to the next result when reading the results of a batch of statements.
		/// </summary>
		/// <returns><see langword="true"/> if there are more result sets; 
		/// otherwise <see langword="false"/>.</returns>
		public override bool NextResult()
		{
			return false;
		}

		/// <summary>
		/// Advances the reader to the next record in a result set.
		/// </summary>
		/// <returns><see langword="true"/> if there are more rows; 
		/// otherwise <see langword="false"/>.</returns>
		public override bool Read()
		{
			if (data==null) {
				data = source.GetData();
				enumerator = data.GetEnumerator();
			}
			return enumerator.MoveNext(); 
		}


		internal void InitForbiddenTypes()
		{
		}

		internal MEScontrolDataReader(IDataReaderSource source, CommandBehavior behavior, Dictionary<string, Type> typeByField)
		{
			this.targetType = source.GetItemType();
			this.behavior = behavior;
			if (typeByField==null || typeByField.Count==0) {
				MemberInfo[] members = PopulateMembers(targetType);
				foreach (MemberInfo member in members)
					expressions.Add(member.Name);
				expressions.Add("This"); 
			}
			else
				foreach (string field in typeByField.Keys)
					expressions.Add(field);
			this.source = source;
			this.typeByField = typeByField;
		}
	}
}
