using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Common;
using System.Data;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents a parameter to a <see cref="MEScontrolCommand"/>.
	/// </summary>
	public class MEScontrolParameter : DbParameter
	{
		private DbType dbType = DbType.Object;
		private object parent;
		private bool isNullable = false;
		private string parameterName;
		private int size = 0;
		private object value;

  
		/// <summary>
		/// Gets or sets the <see cref="DbType"/> of the parameter.
		/// </summary>
		public override DbType DbType {
			get { return dbType; }
			set { dbType = value; }
		}

		/// <summary>
		/// Gets or sets a value that indicates whether the parameter is input-only,
		/// output-only, bidirectional, or a stored procedure return value parameter.
		/// Only <see cref="ParameterDirection.Input"/> is supported by MEScontrolParameter.
		/// </summary>
		public override ParameterDirection Direction {
			get { return ParameterDirection.Input; }
			set {
				if (value!=ParameterDirection.Input) 
					throw new NotSupportedException(TranslationResources.OnlyParameterDirectionInputIsSupported);
			}
		}

		/// <summary>
		/// Gets or sets a value that indicates whether the parameter accepts null values.
		/// </summary>
		public override bool IsNullable {
			get { return isNullable; }
			set { isNullable = value; }
		}

		/// <summary>
		/// Gets or sets the name of the <see cref="MEScontrolParameter"/>.
		/// </summary>
		public override string ParameterName {
			get { return parameterName; }
			set { parameterName = value; }
		}

		/// <summary>
		/// Resets the type associated with this <see cref="MEScontrolParameter"/>. 
		/// </summary>
		public override void ResetDbType()
		{
			DbType = DbType.Object;  
		}

		/// <summary>
		/// Gets or sets the maximum size, in bytes, of the data within the column.
		/// </summary>
		public override int Size {
			get { return size; }
			set { size = value; }
		}

		/// <summary>
		/// Gets or sets the name of the source column mapped to the <see cref="DataSet"/>
		/// and used for loading or returning the <see cref="Value"/>. 
		/// not supported.
		/// </summary>
		public override string SourceColumn {
			get { return ""; }
			set { throw new NotSupportedException(TranslationResources.ThisPropertyIsNotSupported); }
		}

		/// <summary>
		/// Sets or gets a value which indicates whether the source column is nullable.
		/// Not supported.
		/// </summary>
		public override bool SourceColumnNullMapping {
			get { return true; }
			set { throw new NotSupportedException(TranslationResources.ThisPropertyIsNotSupported); }
		}

		/// <summary>
		/// Gets or sets the <see cref="DataRowVersion"/> to use when you load <see cref="Value"/>.
		/// Not supported.
		/// </summary>
		public override DataRowVersion SourceVersion {
			get { return DataRowVersion.Default; }
			set { throw new NotSupportedException(TranslationResources.ThisPropertyIsNotSupported); }
		}

		/// <summary>
		/// Gets or sets the value of the parameter. 
		/// </summary>
		public override object Value	{
			get { return value; }
			set { this.value = value; }
		}

		/// <summary>
		/// Gets or sets the values of the parameter. 
		/// </summary>
		public object[] Values {
			get {
				return value as object[];
			}
			set {
				this.value = value;
			}
		}

		internal object CompareExchangeParent(MEScontrolParameterCollection value, object comparand)
		{
			object obj = this.parent;
			if (comparand==obj)
				this.parent = value;
			return obj;
		}

		internal void ResetParent()
		{
			parent = null;
		}
		
		// Constructors
    
    /// <summary>
    /// Initializes a new instance of this type.
    /// </summary>
    /// <param name="name">Parameter name.</param>
    /// <param name="dbType">Parameter type.</param>
    /// <param name="value">Parameter value.</param>
    public MEScontrolParameter(string name, DbType dbType, object value)
    {
			this.ParameterName = name;
			this.DbType = dbType;
			this.Value = value;
    }
    
    /// <summary>
    /// Initializes a new instance of this type.
    /// </summary>
    /// <param name="name">Parameter name.</param>
    /// <param name="value">Parameter value.</param>
    public MEScontrolParameter(string name, object value)
    {
			this.ParameterName = name;
			this.Value = value;
    }
    
    /// <summary>
    /// Initializes a new instance of this type.
    /// </summary>
    /// <param name="name">Parameter name.</param>
    /// <param name="dbType">Parameter type.</param>
    public MEScontrolParameter(string name, DbType dbType)
    {
			this.ParameterName = name;
			this.DbType = dbType;
    }
    
    /// <summary>
    /// Initializes a new instance of this type.
    /// </summary>
    /// <param name="name">Parameter name.</param>
    public MEScontrolParameter(string name)
    {
			this.ParameterName = name;
    }
    
    /// <summary>
    /// Initializes a new instance of this type.
    /// </summary>
		public MEScontrolParameter()
    {
    }
	}
}
