using System;
using System.Collections.Generic;
using System.Text;
using System.Data.Common;
using System.Collections;
using System.Globalization;
using System.Data;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	/// <summary>
	/// Represents a collection of parameters associated with a
	/// <see cref="MEScontrolCommand"/>. 
	/// </summary>
	public class MEScontrolParameterCollection : DbParameterCollection
	{
		private static Type itemType = typeof(MEScontrolParameter);
		private List<MEScontrolParameter> innerList = new List<MEScontrolParameter>();

		private static int IndexOf(IEnumerable items, string parameterName)
		{
			if (items!=null) {
				int num = 0;
				foreach (MEScontrolParameter parameter2 in items) {
					if (string.Equals(parameterName, parameter2.ParameterName))					
						return num;					
					num++;
				}
				num = 0;
				foreach (MEScontrolParameter parameter in items) {
					if (CultureInfo.CurrentCulture.CompareInfo.Compare(
						parameterName,
						parameter.ParameterName,
						CompareOptions.IgnoreWidth | CompareOptions.IgnoreKanaType | CompareOptions.IgnoreCase)==0)
						return num;					
					num++;
				}
			}
			return -1;
		}

		private void ValidateType(object value)
		{
			if (value==null)
				throw new ArgumentNullException("value");

			if (!itemType.IsInstanceOfType(value))
				throw new ArgumentException(string.Format(TranslationResources.ArgumentValueMustBeInstanceOfType, itemType.Name));
		}

		private void Validate(int index, object value)
		{
			if (value==null)
				throw new ArgumentNullException("value");

			object obj2 = ((MEScontrolParameter)value).CompareExchangeParent(this, null);
			if (obj2!=null)	{
				//if (this!=obj2)
				//	throw new ArgumentException("This parameter already belong another collection.");

				if (index!=this.IndexOf(value))
					throw new ArgumentException(TranslationResources.ThisParameterAlreadyBelongThisCollection);
			}
			if (((MEScontrolParameter)value).ParameterName.Length==0) {
				string parameterName;
				index = 1;
				do {
					parameterName = "Parameter" + index.ToString(CultureInfo.CurrentCulture);
					index++;
				}
				while (-1!=IndexOf(parameterName));
					((MEScontrolParameter)value).ParameterName = parameterName;
			}
		}

		private void RangeCheck(int index)
		{
			if (index<0 || Count<=index)
				throw new ArgumentException(TranslationResources.IndexIsInvalid);
		}

		private int CheckName(string parameterName)
		{
			int index = this.IndexOf(parameterName);
			if (index< 0)
				throw new ArgumentException(TranslationResources.ThereIsNotParameterWithSuchNameInTheCollection);
			return index;
		}

		private void RemoveIndex(int index)
		{
			MEScontrolParameter parameter = innerList[index];
			innerList.RemoveAt(index);
			parameter.ResetParent();
		}

		private void Replace(int index, object newValue)
		{
			ValidateType(newValue);
			Validate(index, newValue);
			MEScontrolParameter parameter = innerList[index];
			innerList[index] = (MEScontrolParameter)newValue;
			parameter.ResetParent();
		}

		/// <summary>
		/// Specifies the number of items in the collection.
		/// </summary>
		public override int Count {
			get {
				if (innerList==null)
					return 0;
				return innerList.Count;
			}
		}

		/// <summary>
		/// Specifies whether the collection is a fixed size.
		/// </summary>
		public override bool IsFixedSize {
			get { return ((IList)innerList).IsFixedSize; }
		}

		/// <summary>
		/// Specifies whether the collection is read-only.
		/// </summary>
		public override bool IsReadOnly {
			get { return ((IList)innerList).IsReadOnly; }
		}

		/// <summary>
		/// Specifies whether the collection is synchronized.
		/// </summary>
		public override bool IsSynchronized {
			get { return ((IList)innerList).IsSynchronized; }
		}

		/// <summary>
		/// Specifies the <see cref="Object"/> to be used to synchronize access to the collection.
		/// </summary>
		public override object SyncRoot {
			get { return ((ICollection)innerList).SyncRoot; }
		}

		/// <summary>
		/// Adds the specified <see cref="MEScontrolParameter"/> object to the 
		/// <see cref="MEScontrolParameterCollection"/>.
		/// </summary>
		/// <param name="value">An <see cref="Object"/>.</param>
		/// <returns>The index of the new <see cref="MEScontrolParameter"/> object.</returns>
		public override int Add(object value)
		{
			ValidateType(value);
			Validate(-1, value);
			innerList.Add((MEScontrolParameter)value);
			return Count-1;
		}

		/// <summary>
		/// Adds the specified <see cref="MEScontrolParameter"/> object to the 
		/// <see cref="MEScontrolParameterCollection"/>.
		/// </summary>
		/// <param name="value">The <see cref="MEScontrolParameter"/> to add to the collection.</param>
		/// <returns>The index of the new <see cref="MEScontrolParameter"/> object.</returns>
		public MEScontrolParameter Add(MEScontrolParameter value)
		{
			Add((object)value);
			return value;
		}

		/// <summary>
		/// Adds a <see cref="MEScontrolParameter"/> to the <see cref="MEScontrolParameterCollection"/>
		/// given the parameter name and the data type.
		/// </summary>
		/// <param name="parameterName">The name of the parameter.</param>
		/// <param name="dbType">One of the <see cref="DbType"/> values.</param>
		/// <returns>The index of the new <see cref="MEScontrolParameter"/> object.</returns>
		public MEScontrolParameter Add(string parameterName, DbType dbType)
		{
			return Add(new MEScontrolParameter(parameterName, dbType));
		}

		/// <summary>
		/// Adds an array of <see cref="MEScontrolParameter"/> values to the end
		/// of the <see cref="MEScontrolParameterCollection"/>.
		/// </summary>
		/// <param name="values">The <see cref="MEScontrolParameter"/> values to add.</param>
		public void AddRange(MEScontrolParameter[] values)
		{
			this.AddRange(values);
		}

		/// <summary>
		/// Adds an array of values to the end of the <see cref="MEScontrolParameterCollection"/>.
		/// </summary>
		/// <param name="values">The <see cref="Array"/> values to add.</param>
		public override void AddRange(Array values)
		{
			if (values == null)
				throw new ArgumentNullException("values");

			foreach (object obj in values) 
				ValidateType(obj);

			foreach (MEScontrolParameter parameter in values) {
				Validate(-1, parameter);
				innerList.Add(parameter);
			}
		}

		/// <summary>
		/// Adds a value to the end of the <see cref="MEScontrolParameterCollection"/>.
		/// </summary>
		/// <param name="parameterName">The name of the parameter.</param>
		/// <param name="value">The value to be added.</param>
		/// <returns>A <see cref="MEScontrolParameter"/> object.</returns>
		public MEScontrolParameter AddWithValue(string parameterName, object value)
		{
			return Add(new MEScontrolParameter(parameterName, value));
		}

		/// <summary>
		/// Removes all the <see cref="MEScontrolParameter"/> objects from
		/// the <see cref="MEScontrolParameterCollection"/>. 
		/// </summary>
		public override void Clear()
		{
			if (innerList!= null) {
				foreach (MEScontrolParameter parameter in innerList)
					parameter.ResetParent();
				innerList.Clear();
			}
		}

		/// <summary>
		/// Determines whether the specified <see cref="MEScontrolParameter"/> is
		/// in this <see cref="MEScontrolParameterCollection"/>. 
		/// </summary>
		/// <param name="value">The <see cref="MEScontrolParameter"/> value.</param>
		/// <returns><see langword="true"/> if the <see cref="MEScontrolParameterCollection"/>
		/// contains the value; otherwise <see langword="false"/>.</returns>
		public bool Contains(MEScontrolParameter value)
		{
			return IndexOf(value)!=-1;
		}

		/// <summary>
		/// Determines whether the specified <see cref="String"/> is
		/// in this <see cref="MEScontrolParameterCollection"/>. 
		/// </summary>
		/// <param name="value">The <see cref="String"/> value.</param>
		/// <returns><see langword="true"/> if the <see cref="MEScontrolParameterCollection"/>
		/// contains the value; otherwise <see langword="false"/>.</returns>
		public override bool Contains(string value)
		{
			return IndexOf(value)!=-1;
		}

		/// <summary>
		/// Determines whether the specified <see cref="Object"/> is
		/// in this <see cref="MEScontrolParameterCollection"/>. 
		/// </summary>
		/// <param name="value">The <see cref="Object"/> value.</param>
		/// <returns><see langword="true"/> if the <see cref="MEScontrolParameterCollection"/>
		/// contains the value; otherwise <see langword="false"/>.</returns>
		public override bool Contains(object value)
		{
			return IndexOf(value)!=-1;
		}

		/// <summary>
		/// Copies all the elements of the current <see cref="MEScontrolParameterCollection"/>
		/// to the specified <see cref="MEScontrolParameterCollection"/> starting at the specified destination index.
		/// </summary>
		/// <param name="array">The <see cref="MEScontrolParameterCollection"/> that is the
		/// destination of the elements copied from the current <see cref="MEScontrolParameterCollection"/>.</param>
		/// <param name="index">A 32-bit integer that represents the index in the
		/// <see cref="MEScontrolParameterCollection"/> at which copying starts.</param>
		public void CopyTo(MEScontrolParameter[] array, int index)
		{
			CopyTo(array, index);
		}

		/// <summary>
		/// Copies all the elements of the current <see cref="MEScontrolParameterCollection"/>
		/// to the specified one-dimensional Array starting at the specified destination Array index.
		/// </summary>
		/// <param name="array">The one-dimensional <see cref="Array"/> that is the destination
		/// of the elements copied from the current <see cref="MEScontrolParameterCollection"/>.</param>
		/// <param name="index">A 32-bit integer that represents the index in the <see cref="Array"/>
		/// at which copying starts.</param>
		public override void CopyTo(Array array, int index)
		{
			((ICollection)innerList).CopyTo(array, index);
		}

		/// <summary>
		/// Returns an enumerator that iterates through the <see cref="MEScontrolParameterCollection"/>. 
		/// </summary>
		/// <returns>An <see cref="IEnumerator"/> for the <see cref="MEScontrolParameterCollection"/>.</returns>
		public override IEnumerator GetEnumerator()
		{
			return innerList.GetEnumerator();
		}

		/// <summary>
		/// Gets the <see cref="DbParameter"/> with the specified name.
		/// </summary>
		/// <param name="parameterName">The name of the <see cref="DbParameter"/> in the collection.</param>
		/// <returns>The <see cref="DbParameter"/> the object with the specified name.</returns>
		protected override DbParameter GetParameter(string parameterName)
		{
			int index = IndexOf(parameterName);
			if (index<0)
				throw new ArgumentException(TranslationResources.ParameterDoesNotFound);
			return innerList[index];
		}

		/// <summary>
		/// Returns the <see cref="DbParameter"/> object at the specified index in the collection.
		/// </summary>
		/// <param name="index">The index of the <see cref="DbParameter"/> in the collection.</param>
		/// <returns>The <see cref="DbParameter"/> object at the specified index in the collection.</returns>
		protected override DbParameter GetParameter(int index)
		{
			RangeCheck(index);
			return innerList[index];
		}

		/// <summary>
		/// Gets the location of the specified <see cref="MEScontrolParameter"/> within the collection.
		/// </summary>
		/// <param name="value">The <see cref="MEScontrolParameter"/> to find.</param>
		/// <returns>The zero-based location of the specified <see cref="MEScontrolParameter"/> that is
		/// a <see cref="MEScontrolParameter"/> within the collection. Returns -1 when the object does
		/// not exist in the <see cref="MEScontrolParameterCollection"/>.</returns>
		public int IndexOf(MEScontrolParameter value)
		{
			return this.IndexOf(value);
		}

		/// <summary>
		/// Gets the location of the specified <see cref="MEScontrolParameter"/> with the specified name.
		/// </summary>
		/// <param name="parameterName">The case-sensitive name of the <see cref="MEScontrolParameter"/>
		/// to find.</param>
		/// <returns>The zero-based location of the specified <see cref="MEScontrolParameter"/> with the
		/// specified case-sensitive name. Returns -1 when the object does not exist in the 
		/// <see cref="MEScontrolParameterCollection"/>.</returns>
		public override int IndexOf(string parameterName)
		{
			return IndexOf(innerList, parameterName);
		}

		/// <summary>
		/// Gets the location of the specified <see cref="Object"/> within the collection. 
		/// </summary>
		/// <param name="value">The <see cref="Object"/> to find.</param>
		/// <returns>The zero-based location of the specified <see cref="Object"/> that is
		/// a <see cref="MEScontrolParameter"/> within the collection. Returns -1 when the
		/// object does not exist in the <see cref="MEScontrolParameterCollection"/>.</returns>
		public override int IndexOf(object value)
		{
			if (value!=null) {
				ValidateType(value);
				if (innerList!=null) {
					int count = innerList.Count;
					for (int i=0;i<count;i++)
						if (value==innerList[i])
							return i;
				}
			}
			return -1;
		}

		/// <summary>
		/// Inserts a <see cref="MEScontrolParameter"/> object into the
		/// <see cref="MEScontrolParameterCollection"/> at the specified index.
		/// </summary>
		/// <param name="index">The zero-based index at which value should be inserted.</param>
		/// <param name="value">A <see cref="MEScontrolParameter"/> object to be inserted
		/// in the <see cref="MEScontrolParameterCollection"/>.</param>
		public void Insert(int index, MEScontrolParameter value)
		{
			this.Insert(index, value);
		}

		/// <summary>
		/// Inserts an <see cref="Object"/> into the <see cref="MEScontrolParameterCollection"/>
		/// at the specified index.
		/// </summary>
		/// <param name="index">The zero-based index at which value should be inserted.</param>
		/// <param name="value">An <see cref="Object"/> to be inserted in the
		/// <see cref="MEScontrolParameterCollection"/>.</param>
		public override void Insert(int index, object value)
		{
			ValidateType(value);
			Validate(-1, (MEScontrolParameter)value);
			innerList.Insert(index, (MEScontrolParameter)value);
		}

		/// <summary>
		/// Removes the specified <see cref="MEScontrolParameter"/> from the collection.
		/// </summary>
		/// <param name="value">A <see cref="MEScontrolParameter"/> object to remove from
		/// the collection.</param>
		public void Remove(MEScontrolParameter value)
		{
			Remove(value);
		}

		/// <summary>
		/// Removes the specified <see cref="Object"/> from the collection.
		/// </summary>
		/// <param name="value">A <see cref="Object"/> object to remove from the collection.</param>
		public override void Remove(object value)
		{
			ValidateType(value);
			int index = IndexOf(value);
			if (index!=-1)			
				RemoveIndex(index);
			else if (this!=((MEScontrolParameter)value).CompareExchangeParent(null, this))
				throw new ArgumentException(TranslationResources.ThisObjectCanNotBeRemoved);
		}

		/// <summary>
		/// Removes the <see cref="MEScontrolParameter"/> from the
		/// <see cref="MEScontrolParameterCollection"/> at the specified parameter name.
		/// </summary>
		/// <param name="parameterName">The name of the <see cref="MEScontrolParameter"/>
		/// to remove.</param>
		public override void RemoveAt(string parameterName)
		{
			int index = CheckName(parameterName);
			RemoveIndex(index);
		}

		/// <summary>
		/// Removes the <see cref="MEScontrolParameter"/> from the 
		/// <see cref="MEScontrolParameterCollection"/> at the specified index.
		/// </summary>
		/// <param name="index">The zero-based index of the <see cref="MEScontrolParameter"/>
		/// object to remove.</param>
		public override void RemoveAt(int index)
		{
			RangeCheck(index);
			RemoveIndex(index);
		}

		/// <summary>
		/// Sets the <see cref="DbParameter"/> object with the specified name to a new value.
		/// </summary>
		/// <param name="parameterName">The name of the <see cref="DbParameter"/> object in the
		/// collection.</param>
		/// <param name="value">The new <see cref="DbParameter"/> value.</param>
		protected override void SetParameter(string parameterName, DbParameter value)
		{
			int index = IndexOf(parameterName);
			if (index<0)
				throw new ArgumentException(TranslationResources.ParameterDoesNotFound);
			Replace(index, value);
		}

		/// <summary>
		/// Sets the <see cref="DbParameter"/> object at the specified index to a new value. 
		/// </summary>
		/// <param name="index">The index where the <see cref="DbParameter"/> object is located.</param>
		/// <param name="value">The new <see cref="DbParameter"/> value.</param>
		protected override void SetParameter(int index, DbParameter value)
		{
			RangeCheck(index);
			Replace(index, value);
		}

		internal MEScontrolParameterCollection()
		{ 
		}
	}
}
