using System;
using System.Collections.Generic;
using System.Text;
using System.Collections;
using System.Reflection;
using Mesware.MesControl.DataProvider.Properties;

namespace Mesware.MesControl.DataProvider
{
	public abstract class MethodDataReaderSource : IDataReaderSource
	{		
		private string commandText;
		private System.Type itemType;
		private IEnumerable data;
		private MEScontrolParameterCollection parameters;		
		private string[] arguments;
		private object targetObject;

		private Type GetGenericArgumentForBaseType(Type type, Type baseType)
		{
			if (type==null)
				return null;
			if (type.IsGenericType && type.GetGenericTypeDefinition()==baseType)
				return type.GetGenericArguments()[0];
			else {
				Type result = GetGenericArgumentForBaseType(type.BaseType, baseType);
				if (result!=null)
					return result;
				foreach (Type interfaceType in type.GetInterfaces()) {
					result = GetGenericArgumentForBaseType(interfaceType, baseType);
					if (result!=null)
						return result;
				}
			}
			return null;
		}

		protected abstract void ProcessCommand(
			string commandText, 
			out string newCommandText, 
			out object targetObject, 
			out string methodName,
			out string[] parameters);

		private void EnsureItemType()
		{
			if (itemType!=null)
				return;

			string newCommandText;
			string methodName;
			ProcessCommand(
				commandText,
				out newCommandText,
				out targetObject, 
				out methodName,
				out arguments);

			commandText = newCommandText;

			MethodInfo[] methods = (MethodInfo[])targetObject.GetType().GetMember(
				methodName,
				MemberTypes.Method,
				BindingFlags.Instance | BindingFlags.Public);
			Type resultType = null;
			if (methods.Length==0)
				throw new InvalidOperationException(string.Format(TranslationResources.CanNotFindMethodWithName, methodName));			

			foreach(MethodInfo method in methods)
				if (method.GetParameters().Length==arguments.Length) {
					resultType = method.ReturnType;
					break;
				}
						
			if (resultType==null)
				throw new InvalidOperationException(string.Format(TranslationResources.CanNotFindMethodWithNameAndSuitableParameters, methodName));
			if (resultType.IsArray)
				itemType = resultType.GetElementType();			
			else {
				Type genericEnumerableType = GetGenericArgumentForBaseType(resultType, typeof(IEnumerable<>));
				if (genericEnumerableType!=null)
					itemType = genericEnumerableType;
				else if (resultType==typeof(IEnumerable) || resultType.IsSubclassOf(typeof(IEnumerable)))
						itemType = typeof(object);
			}
			if (itemType==null)
				itemType = resultType;				
		}

		private void EnsureData()
		{
			if (data!=null)
				return;
			EnsureItemType();			
			Hashtable evalParams = new Hashtable(arguments.Length+1);
			commandText = "{__targetObject}."+commandText;
			evalParams["__targetObject"] = targetObject;

			foreach (string arg in arguments) {
				string targ = arg.Trim();
				if (targ=="")
					continue;
				if (targ[0]=='@') {
					string paramName = targ.Substring(1);
					if (!parameters.Contains(targ))
						throw new InvalidOperationException(string.Format(TranslationResources.ParameterWithNameIsNotFound, paramName));
					object paramValue = parameters[targ].Value;
					evalParams.Add(paramName, paramValue);
					commandText = commandText.Replace(targ, "{"+paramName+"}"); 
				}
			}

			object result =null;

			try {
				result = Evaluator.Eval(commandText, evalParams);
			}
			catch (TargetInvocationException ex) {
				if (ex.InnerException!=null)
					throw ex.InnerException;
				else
					throw;
			}
			if (result is IEnumerable)
				data = result as IEnumerable;
			else
				data = new object[] { result };
		}

		#region IDataReaderSource Members

		/// <summary>
		/// Gets the <see cref="Type"/> of item.
		/// </summary>
		/// <returns><see cref="Type"/> of item.</returns>
		public Type GetItemType()
		{
			EnsureItemType();
			return itemType;
		}

		/// <summary>
		/// Gets data of source.
		/// </summary>
		/// <returns>Data of source.</returns>
		public IEnumerable GetData()
		{
			EnsureData();
			return data;
		}

		/// <summary>
		/// Sets the evalParams for source.
		/// </summary>
		/// <param name="parameters">Parameters for source.</param>
		public void SetParameters(MEScontrolParameterCollection parameters)
		{
			this.parameters = parameters;
		}

		#endregion

		/// <summary>
		/// Initializes a new instance of the <see cref="QueryDataReaderSource"/> class.
		/// </summary>
		/// <param name="session">Current <see cref="Session"/>.</param> 
		/// <param name="commandText">Command text of the source.</param>
		public MethodDataReaderSource(string commandText)
		{			
			this.commandText = commandText; 
		}
	}
}
